<?
/**
 * A class for creating the Google Checkout forms for selling items from your own website
 * @author Greg Neustaetter [www.gregphoto.net]
 * @version 0.1
 *
 */
class Gregphoto_Service_Google_Checkout {
	protected $merchant_id;
	protected $merchant_key;
	protected $xml;
	protected $items = array();
	protected $flatRateShipping = array();
	protected $pickupShipping = array();
	protected $merchantCalcShipping = array();
	protected $defaultCurrency = 'USD';
	protected $stateTax = array();
	protected $defaultXML = false;
	protected $urlPrefix = 'https://checkout.google.com/';
	/**
	 * Url the user will be sent to if they choose to edit their cart
	 *
	 * @var string
	 */
	public $editCartUrl = false;
	/**
	 * Url the user will be sent to if they choose to continue shopping after purchasing the items in the cart
	 *
	 * @var string
	 */
	public $continueShoppingUrl = false;
	/**
	 * Whether or not shipping should be taxed
	 *
	 * @var Boolean
	 */
	public $shippingTaxed = false;

	/**
	 * Instantiates the class
	 *
	 * @param string $merchant_id The ten or 15 digit merchant ID assigned by Google
	 * @param string $merchant_key The twenty-two character merchant key assigned by Google
	 */
	public function __construct($merchant_id,$merchant_key) {
		if((strlen($merchant_id) != 10) && (strlen($merchant_id) != 15)) {
			throw new Exception("Merchant ID must be a 10 or 15 digit number");
		}
		if(strlen($merchant_key) != 22) {
			throw new Exception("Merchant Key must be a 22 character string");
		}
		$this->merchant_id = $merchant_id;
		$this->merchant_key = $merchant_key;
	}

	/**
	 * Add a new item to the cart
	 *
	 * @param string $name The part name/number
	 * @param string $description The part description
	 * @param double $price The unit price of the item in the default currency.  May be positive or negative.
	 * @param int $quantity The quantity of the item.
	 * @param string $privateData Additional data to include with the item.  This will not be shown to the user checking out.
	 * @param string $taxTable The alternative tax table to be used when computing tax for this item
	 */
	public function addItem($name,$description,$price,$quantity,$privateData = false,$taxTable = false) {
		$requiredList = array('name','description','price','quantity');
		foreach($requiredList as $required) {
			if(empty($$required)) {
				throw new Exception("{$required} is a required field when adding items");
			}
		}
		$name = htmlspecialchars($name);
		$description = htmlspecialchars($description);
		$price = htmlspecialchars($price);
		$quantity = htmlspecialchars($quantity);
		$this->items[] = array('name'=>$name,'description'=>$description,'price'=>$price,'quantity'=>$quantity,'taxTable'=>$taxTable,'privateData'=>$privateData);
	}

	/**
	 * Creates the cart XML
	 *
	 */
	protected function generateXML() {
		if($this->defaultXML) {
			$func = ($this->defaultXML['type'] == 'file') ? 'load' : 'loadXML';
			$this->xml = DOMDocument::$func($this->defaultXML['value']);
			$itemsList = $this->xml->getElementsByTagName('items');
			$itemsNode = $itemsList->item(0);
			$this->addItemsXML($itemsNode);
			$this->xml->formatOutput = true;
			return;
		}
		$this->xml = new DOMDocument('1.0','UTF-8');
		$this->xml->formatOutput = true;
		$checkoutShoppingCart = new DOMElement('checkout-shopping-cart', NULL, 'http://checkout.google.com/schema/2');
		$this->xml->appendChild($checkoutShoppingCart);
		$items = $checkoutShoppingCart->appendChild(new DOMElement('shopping-cart'))->appendChild(new DOMElement('items'));
		$this->addItemsXML($items);


		$checkoutFlowSupport = new DOMElement('checkout-flow-support');
		$checkoutShoppingCart->appendChild($checkoutFlowSupport);
		$merchantOpts = $checkoutFlowSupport->appendChild(new DOMElement('merchant-checkout-flow-support'));
		if($this->editCartUrl) {
			$merchantOpts->appendChild(new DOMElement('edit-cart-url',htmlspecialchars($this->editCartUrl)));
		}
		if($this->continueShoppingUrl) {
			$merchantOpts->appendChild(new DOMElement('continue-shopping-url',htmlspecialchars($this->continueShoppingUrl)));
		}

		// Shipping related
		if((count($this->flatRateShipping) > 0) || (count($this->pickupShipping) > 0) || (count($this->merchantCalcShipping) > 0) ) {
			$shipping = $merchantOpts->appendChild(new DOMElement('shipping-methods'));
			foreach($this->flatRateShipping as $shipInfo) {
				$ship = $shipping->appendChild(new DOMElement('flat-rate-shipping'));
				$ship->setAttribute('name',$shipInfo['name']);
				$price = $ship->appendChild(new DOMElement('price',$shipInfo['price']));
				$price->setAttribute('currency',$this->defaultCurrency);
			}
			foreach($this->pickupShipping as $shipInfo) {
				$ship = $shipping->appendChild(new DOMElement('pickup'));
				$ship->setAttribute('name',$shipInfo['name']);
				$price = $ship->appendChild(new DOMElement('price',$shipInfo['price']));
				$price->setAttribute('currency',$this->defaultCurrency);
			}
		}

		// Tax Related
		if(count($this->stateTax) > 0) {
			$taxTables = $merchantOpts->appendChild(new DOMElement('tax-tables'));
			$defaultRules = $taxTables->appendChild(new DOMElement('default-tax-table'))->appendChild(new DOMElement('tax-rules'));
			foreach($this->stateTax as $taxInfo) {
				$rule = $defaultRules->appendChild(new DOMElement('default-tax-rule'));
				if($this->shippingTaxed) {
					$rule->appendChild(new DOMElement('shipping-taxed','true'));
				}
				$rule->appendChild(new DOMElement('rate',$taxInfo['rate']));
				$rule->appendChild(new DOMElement('tax-area'))->appendChild(new DOMElement('us-state-area'))->appendChild(new DOMElement('state',$taxInfo['state']));
			}
			$alternateTaxTable = $taxTables->appendChild(new DOMElement('alternate-tax-tables'))->appendChild(new DOMElement('alternate-tax-table'));
			$alternateTaxTable->setAttribute('name','taxfree');
			$alternateTaxTable->setAttribute('standalone','true');

			$rule = $alternateTaxTable->appendChild(new DOMElement('alternate-tax-rules'))->appendChild(new DOMElement('alternate-tax-rule'));
			$rule->appendChild(new DOMElement('rate',0));
			$countryArea = $rule->appendChild(new DOMElement('tax-area'))->appendChild(new DOMElement('us-country-area'));
			$countryArea->setAttribute('country-area','ALL');
		}
		return;
	}

	/**
	 * Adds item elements to the XML for all items added with the addItem method
	 *
	 * @param DOMElement $itemsNode The <items> node of the XML
	 */
	protected function addItemsXML($itemsNode) {
		foreach($this->items as $itemInfo) {
			$item = new DOMElement('item');
			$itemsNode->appendChild($item);
			$item->appendChild(new DOMElement('item-name',$itemInfo['name']));
			$item->appendChild(new DOMElement('item-description',$itemInfo['description']));
			$price = new DOMElement('unit-price',$itemInfo['price']);
			$item->appendChild($price);
			$price->setAttribute('currency',$this->defaultCurrency);
			$item->appendChild(new DOMElement('quantity',$itemInfo['quantity']));
			if($itemInfo['taxTable']) {
				$item->appendChild(new DOMElement('tax-table-selector',$itemInfo['taxTable']));
			}
			if($itemInfo['privateData']) {
				$item->appendChild(new DOMElement('merchant-private-item-data'))->appendChild(new DOMElement('item-note',$itemInfo['privateData']));
			}
		}
	}

	/**
	 * Sets the currency for the cart.  The default currency is USD.  You do not need to call this unless you are using a different currency
	 *
	 * @param string $currency The currency code for the cart.
	 */
	public function setCurrency($currency) {
		$this->defaultCurrency = $currency;
	}

	/**
	 * Adds a general shipping rule.  You may add multiple rules of which the user may choose one
	 *
	 * @param string $name The name that will be shown to the user
	 * @param double $price The price of this shipping option.  This is the price for shipping the entire contents of the cart.
	 * @param string $type Available options are 'flat', 'pickup', or 'merchantCalc'.  MerchantCalc is not currently operational.
	 * @param array $options Adds additional options to the rule - this doesn't currently do anything.
	 */
	public function setShipping($name,$price,$type='flat',$options = array()) {
		$name = htmlspecialchars($name);
		switch($type) {
			case 'flat':
				$this->flatRateShipping[] = array('name'=>$name,'price'=>$price);
				break;
			case 'pickup':
				$this->pickupShipping[] = array('name'=>$name,'price'=>$price);
				break;
			case 'merchantCalc':
				$this->merchantCalcShipping[] = array('name'=>$name,'price'=>$price);
				break;
		}
	}

	/**
	 * Sets a tax rate for a given state
	 *
	 * @param string $state The two letter state code, i.e. 'CA'
	 * @param double $rate The tax rate, for example .0875 would be 8.75% tax rate
	 */
	public function setStateTax($state, $rate) {
		$this->stateTax[] = array('state'=>$state, 'rate'=>$rate);
	}

	/**
	 * Returns the HTML checkout button and form code with the cart XML and signature.  This should be called last.
	 *
	 * @return string The HTML code for the form
	 */
	public function getHTMLForm() {
		ob_start();
		?>
		<form action="<?=$this->getURL();?>" method="POST">
			<input type="hidden" name="cart" value="<?=base64_encode($this->getXML());?>">
			<input type="hidden" name="signature" value="<?=base64_encode($this->getSignature());?>">
			<input type="image" name="Google Checkout" alt="Fast checkout through Google"   src="http://checkout.google.com/buttons/checkout.gif?merchant_id=<?=$this->merchant_id;?>&w=180&h=46&style=white&variant=text&loc=en_US" height="46" width="180">
		</form>
		<?
		$html = ob_get_clean();
		return $html;
	}

	/**
	 * Sets the mode of the operation to use either the sandbox or live Google Checkout site.  You only need to call this if using the sandbox as the class defaults to using the live site.
	 *
	 * @param string $mode Either 'sandbox' or 'checkout'.
	 */
	public function setMode($mode) {
		if($mode == 'sandbox') {
			$this->urlPrefix = 'https://sandbox.google.com/';
		}
	}

	/**
	 * Gets the URL for sending the request to Google Checkout
	 *
	 * @return string The url
	 */
	public function getURL() {
		return $this->urlPrefix . "cws/v2/Merchant/". $this->merchant_id . "/checkout";
	}

	/**
	 * Allows the user to specify a base XML file or string that will be used instead of generating the entire string.  The user can then add items to this XML using the addItem method.
	 *
	 * @param string $value The file path or the XML string
	 * @param string $type Either 'file' or 'string', defaults to 'file'.  If file, the value argument must be the path to the file and the file must be readable by the webserver.
	 */
	public function setDefaultXML($value, $type='file') {
		$this->defaultXML = array('type'=>$type,'value'=>$value);
		if(($type == 'file') && !(is_readable($value))) {
			throw new Exception("The XML file specified, '$value', could not be read by the system or does not exist");
		}
	}

	/**
	 * Gets a string representation of the cart XML
	 *
	 * @return string The XML string
	 */
	public function getXML() {
		$this->generateXML();
		return $this->xml->saveXML();
	}

	/**
	 * For debugging, prints the generated XML to the screen
	 *
	 */
	public function printXML() {
		echo "<pre>";
		echo htmlspecialchars($this->getXML());
		echo "</pre>";
	}

	/**
	 * Implements the HMAC_SHA1 encryption
	 *
	 * @param string $data The data to be encrypted
	 * @param string $key The key for encryption
	 * @return string The encrypted data, in a binary format
	 */
	public function HMAC_SHA1($data,$key) {
		$blocksize = 64;
		if (strlen($key) > $blocksize) {
			$key = pack('H*', sha1($key));
		}
		$key = str_pad($key, $blocksize, chr(0x00));
		$ipad = str_repeat(chr(0x36), $blocksize);
		$opad = str_repeat(chr(0x5c), $blocksize);
		$hmac = pack(
						'H*', sha1(
								($key^$opad).pack(
										'H*', sha1(
												($key^$ipad).$data
										)
								)
						)
					);
		return $hmac;
	}

	/**
	 * Get the HMAC_SHA1 encoded signature required for communicating with Google Checkout
	 *
	 * @return string The signature
	 */
	public function getSignature() {
		return $this->HMAC_SHA1($this->getXML(),$this->merchant_key);
	}
}
?>